"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const no_orm_error_1 = require("../errors/no-orm.error");
const typedi_1 = require("typedi");
const dynamic_object_factory_1 = require("../graphql/dynamic-object.factory");
const auto_relay_config_service_1 = require("./../services/auto-relay-config.service");
function RelayedConnection(type, throughOrOptions, options) {
    return (target, propertyKey) => {
        let through;
        if (typeof throughOrOptions === "function") {
            through = throughOrOptions;
        }
        else {
            options = throughOrOptions;
        }
        process.nextTick(() => {
            const getterName = `relayField${String(propertyKey)}Getter`;
            const capitalized = String(propertyKey).charAt(0).toUpperCase() + String(propertyKey).slice(1);
            const dynamicObjectFactory = typedi_1.Container.get(dynamic_object_factory_1.DynamicObjectFactory);
            const { Connection } = dynamicObjectFactory.getEdgeConnection(type, through, options);
            dynamicObjectFactory.declareFunctionAsRelayInSDL(target, getterName, propertyKey, Connection, options);
            try {
                const orm = typedi_1.Container.get(auto_relay_config_service_1.ORM_CONNECTION);
                target[getterName] = orm.autoRelayFactory(String(propertyKey), () => target, type, through, options);
            }
            catch (e) {
                if (e && e.name === "ServiceNotFoundError") {
                    target[getterName] = () => { throw new no_orm_error_1.NoORMError(); };
                }
                else {
                    throw e;
                }
            }
        });
    };
}
exports.RelayedConnection = RelayedConnection;
//# sourceMappingURL=relayed-connection.decorator.js.map