"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const limit_offset_decorator_1 = require("./../decorators/limit-offset.decorator");
const pagination_args_from_data_function_1 = require("../helpers/pagination-args-from-data.function");
const augment_connection_function_1 = require("../helpers/augment-connection.function");
const Relay = __importStar(require("graphql-relay"));
const bi_directional_pageinfo_function_1 = require("../helpers/bi-directional-pageinfo.function");
function RelayFromArrayCountFactory(prototype, propertyKey) {
    return async (resolverData, next) => {
        const [entityCount, entities, throughKey] = extractGetterFunctionReturn(await next());
        const args = pagination_args_from_data_function_1.connectionArgsFromResolverData(prototype, propertyKey, resolverData);
        const pagination = limit_offset_decorator_1.RelayLimitOffsetResolverFactory(prototype, propertyKey)(resolverData);
        const pageMeta = { sliceStart: pagination.offset || 0, arrayLength: entityCount };
        const rawConnection = Relay.connectionFromArraySlice(entities, args, pageMeta);
        let connection = rawConnection;
        const options = JSON.parse(Reflect.getMetadata('autorelay:query:options', prototype, propertyKey) || '{}');
        if (options.biDirectionalPageInfo) {
            connection = bi_directional_pageinfo_function_1.biDirectionalPageInfo(rawConnection, pageMeta, entities.length);
        }
        const metadatas = { _autoRelayMetadata: { totalItems: entityCount } };
        Object.assign(connection, metadatas);
        Object.assign(connection.pageInfo, metadatas);
        if (throughKey) {
            return augment_connection_function_1.augmentedConnection(connection, String(throughKey));
        }
        else {
            return connection;
        }
    };
}
exports.RelayFromArrayCountFactory = RelayFromArrayCountFactory;
function extractGetterFunctionReturn(payload) {
    if (Array.isArray(payload)) {
        if (typeof payload[0] === "number") {
            return payload;
        }
        else if (typeof payload[1] === "number") {
            const [models, count, key] = payload;
            return [count, models, key];
        }
        else {
            throw new WrongReturnType(`got [${typeof payload[0]}, ${typeof payload[1]}, ${typeof payload[2]}] instead of expected [number, entity[]] or [entity[], number] }`);
        }
    }
    else {
        const { count, rows, key } = payload;
        if (typeof count !== "number")
            throw new WrongReturnType(`count is supposed to be number, got ${count} instead`);
        if (!Array.isArray(rows))
            throw new WrongReturnType(`rows is supposed to be array, got ${rows} instead`);
        return [count, rows, key];
    }
}
exports.extractGetterFunctionReturn = extractGetterFunctionReturn;
class WrongReturnType extends Error {
    constructor(message) {
        super(`Wrong return type for Relayed method: ${message}`);
    }
}
exports.WrongReturnType = WrongReturnType;
//# sourceMappingURL=relay-from-array-count.middleware.js.map