"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const typedi_1 = require("typedi");
const Relay = __importStar(require("graphql-relay"));
let LimitOffsetPagingService = class LimitOffsetPagingService {
    getLimitOffset(args) {
        const meta = this._checkPagingSanity(args);
        switch (meta.pagingType) {
            case 'forward': {
                return {
                    limit: meta.first,
                    offset: meta.after ? this._nextId(meta.after) : 0,
                };
            }
            case 'backward': {
                const { last, before } = meta;
                let limit = last;
                let offset = this._getId(before) - last;
                if (offset < 0) {
                    limit = Math.max(last + offset, 0);
                    offset = 0;
                }
                return { offset, limit };
            }
            default:
                return {};
        }
    }
    _checkPagingSanity(args) {
        const after = args.after || undefined;
        const before = args.before || undefined;
        const first = args.first || 0;
        const last = args.last || 0;
        const isForwardPaging = !!first || !!after;
        const isBackwardPaging = !!last || !!before;
        if (isForwardPaging && isBackwardPaging) {
            if ((isForwardPaging && before) || (isBackwardPaging && after)) {
                throw new Error('paging must use either first/after or last/before');
            }
            else {
                throw new Error('cursor-based pagination cannot be forwards AND backwards');
            }
        }
        if ((isForwardPaging && first < 0) || (isBackwardPaging && last < 0)) {
            throw new Error('paging limit must be positive');
        }
        if (last && !before) {
            throw new Error('when paging backwards, a \'before\' argument is required');
        }
        if (isForwardPaging) {
            return { pagingType: 'forward', after, first };
        }
        else if (isBackwardPaging) {
            return { pagingType: 'backward', before: before, last };
        }
        else {
            return { pagingType: 'none' };
        }
    }
    _getId(cursor) {
        return parseInt(Relay.fromGlobalId(cursor).id, 10);
    }
    _nextId(cursor) {
        return this._getId(cursor) + 1;
    }
};
LimitOffsetPagingService = __decorate([
    typedi_1.Service()
], LimitOffsetPagingService);
exports.LimitOffsetPagingService = LimitOffsetPagingService;
;
//# sourceMappingURL=limit-offset-paging.service.js.map